import requests
import json

from ..MuviBase import MuviBase


def _format(response):
    return {
        'email': response['email'],
        'first_name': response['nombre'],
        'last_name': response['apellido'],
    }


def _check(client: dict) -> dict:
    lista = {
        'nombre': None,
        'apellido': None,
        'celular': None,
        'email': None
    }
    for k in lista.keys():
        if k not in client:
            client[k] = lista[k]
    if 'domicilio' not in client:
        client['domicilio'] = {
            'altura': '1',
            'calle': '-'
        }
    else:
        if 'calle' not in client['domicilio']:
            client['domicilio']['calle'] = '-'
        if 'altura' not in client['domicilio']:
            client['domicilio']['altura'] = '1'

    return client


class Customer(MuviBase):
    def __init__(self, processor: str, url: str, private_key: str, public_key: str):
        super().__init__(processor)
        self.url = url
        self.private_key = private_key
        self.public_key = public_key
        self.headers = {
            'apikey': self.private_key,
            'Content-Type': 'application/json',
            'Cache-Control': 'no-cache'
        }

    def create(self, client: dict) -> dict:
        client = client.copy()
        client = _check(client)
        formatted_client = _format(client)
        customer = {
            'customer': {
                'name': '{} {}'.format(client['nombre'], client['apellido']),
                'identification': {
                    'type': 'dni',
                    'number': client['documento']
                }
            }
        }
        self.headers['apikey'] = self.public_key
        r = requests.post(self.url + '/tokens', headers=self.headers, data=json.dumps(customer))
        self.headers['apikey'] = self.private_key
        formatted_client['id'] = r.json()['id']
        if r.status_code > 400:
            return self.error(response=r.json(), status=r.status_code)
        return self.ok(response=formatted_client, status=r.status_code)

    def get(self, customer_id: str) -> dict:
        # No existen los datos del customer almacenados en decidir
        return self.error(message='Customer not found')

    def search(self, filters: dict):
        return self.error(message='Customer not found')



import mercadopago

from ..MuviBase import MuviBase


def _format(response):
    return {
        'id': response['id'],
        'card_type': response['payment_method']['payment_type_id'],
        'card_brand': response["payment_method"]["name"],
        'last_four_digits': response['last_four_digits'],
        'issuer': {
            'name': response['issuer']['name']
        },
        'cardholder': {
            'name': response['cardholder']['name']
        },
        'expiration_month': response['expiration_month'],
        'expiration_year': response['expiration_year'],
        'payer_id': response['user_id']
    }


class Card(MuviBase):
    def __init__(self, processor: str, sdk: mercadopago.SDK):
        super().__init__(processor)
        self.sdk = sdk

    def create(self, customer_id: str, card_token: str):
        card_data = {'token': card_token}
        mp_response = self.sdk.card().create(customer_id, card_data)

        if mp_response['status'] < 400:
            return self.ok(response=_format(mp_response['response']), status=mp_response['status'])
        return self.error(message=mp_response['response']['error'], status=mp_response['status'])

    def get(self, customer_id: str, card_id: str) -> dict:
        mp_response = self.sdk.card().get(customer_id, card_id)
        if mp_response['status'] < 400:
            card = mp_response['response']
            return self.ok(response=_format(card), status=mp_response['status'])
        return self.error(message=mp_response['response']['message'], status=mp_response['status'])

    def list_all(self, customer_id: str):
        mp_response = self.sdk.card().list_all(customer_id)
        if mp_response['status'] > 299:
            return self.error(message='Customer not found', status=mp_response['status'])
        cards = mp_response['response']
        return self.ok(response={'results': [_format(card) for card in cards]}, status=mp_response['status'])



from .mercadopago_sdk.MercadoPagoSDK import MercadoPagoSDK
from .decidir_sdk.DecidirSDK import DecidirSDK


class SDK:
    def __init__(self, merchant: dict, merchant_cadena: dict, processor: str = None, marketplace: bool = False):
        # Si se le indica el procesador
        self._sdk = None
        if processor == 'mercadopago':
            self.processor = 'mercadopago'
            self._sdk = MercadoPagoSDK(merchant, marketplace=marketplace)
        elif processor == 'decidir':
            self.processor = 'decidir'
            self._sdk = DecidirSDK(merchant, merchant_cadena, marketplace=marketplace)

    def customer(self):
        return self._sdk.customer()

    def card(self):
        return self._sdk.card()

    def card_token(self):
        return self._sdk.card_token()

    def payment(self):
        return self._sdk.payment()

    def refund(self):
        return self._sdk.refund()

    def ok(self):
        return self._sdk is not None and self._sdk.ok()



import requests
import random
import string
from iso8601 import parse_date
import pytz
from ..MuviBase import MuviBase

map_errors = {
    1:"pedir autorizacion",
    2: "pedir autorizacion",
    3: "comercio invalido",
    4: "capturar tarjeta",
    5: "denegada",
    7: "retenga y llame",
    8: "ingreso manual incorrecto",
    12: "transac. invalida",
    13: "monto invalido",
    14: "tarjeta invalida",
    19: "auto approve in receivepayment",
    25: "no existe original",
    28: "servicio no disponible",
    30: "error en formato",
    31: "the parameter must be specified.",
    38: "excede ing.de pin",
    39: "ingreso manual incorrecto",
    43: "retener tarjeta",
    45: "no opera en cuotas",
    46: "tarjeta no vigente",
    47: "pin requerido",
    48: "excede max. cuotas",
    49: "error fecha vencim.",
    50: "entrega supera limite",
    51: "fondos insuficientes",
    53: "cuenta inexistente",
    54: "tarjeta vencida",
    55: "pin incorrecto",
    56: "tarj. no habilitada",
    57: "trans. no permitida",
    58: "servicio invalido",
    61: "excede limite",
    65: "excede lim. tarjeta",
    76: "llamar al emisor",
    77: "error plan/cuotas",
    89: "terminal invalida",
    91: "emisor fuera linea",
    94: "nro. sec. duplicad",
    95: "re-transmitiendo",
    96: "error en sistema",
    97: "error en sistema host",
    98: "ver rechazo en ticket",
}
arg_tz = pytz.timezone('America/Buenos_Aires')


def _format_card(response):
    if 'card' in response:
        return {
            'id': response['card']['id'],
            'card_type': 'not_found',
            'last_four_digits': response['card']['last_four_digits'],
            'cardholder': {
                'name': response['card']['cardholder']['name']
            },
            'expiration_month': response['card']['expiration_month'],
            'expiration_year': response['card']['expiration_year'],
        }
        
    if 'card_data' in response and 'tokens' not in response['card_data']:
        return {
            'id': '',
            'card_type': '',
            'last_four_digits': response['card_data']['card_number'][-4:],
            'cardholder': {
                'name': response['card_data']['card_holder']['name']
            },
            'expiration_month': '',
            'expiration_year': '',
        }
    
    return {
        'id': '',
        'card_type': '',
        'last_four_digits': '',
        'cardholder': {
            'name': ''
        },
        'expiration_month': '',
        'expiration_year': '',
    }


def _format(response):
    if response['status'] in ['accredited', 'approved']:
        response['status'] = 'approved'
        response['status_detail'] = 'accredited'
    elif response['status'] == 'rejected':
        id_error = int(response['status_details']['error']['reason']['id'])
        if id_error in map_errors:
            response['status_detail'] = map_errors[id_error]
        else:
            response['status_detail'] = 'error desconocido'
    elif response['status'] == 'annulled' or response['status'] == 'refunded':
        response['status'] = 'refunded'
        response['status_detail'] = 'refunded'
    date = parse_date(response['date']).astimezone(arg_tz)
    result = {
        'id': response['site_transaction_id'],
        'payment_id': response['id'],
        'transaction_amount': response['amount'] / 100,
        'date_created': date,
        'status': response['status'],
        'status_detail': response['status_detail'],
        'card': _format_card(response),
        'payment_day': date,
        'statement_descriptor': response['establishment_name'],
        'payment_method_id': response['payment_method_id'],
        'installments': response['installments'],
    }
    return result


class Payment(MuviBase):
    def __init__(self, processor: str, url: str, private_key: str, public_key: str, merchant_name: str, site_id: str,
                 site_id_cadena: str, marketplace: bool):
        super().__init__(processor)
        self.url = url
        self.private_key = private_key
        self.public_key = public_key
        self.merchant_name = merchant_name
        self.site_id = site_id
        self.site_id_cadena = site_id_cadena
        self.marketplace = marketplace
        self.headers = {
            'apikey': self.private_key,
            'Content-Type': 'application/json',
            'Cache-Control': 'no-cache'
        }

    def create(self, payment_data: dict):
        site_transaction_id = ''.join(random.choices(string.ascii_letters + string.digits, k=10))
        total_amount = round(payment_data['transaction_amount'], 2) * 100  # se multiplica por 100; no acepta decimales
        installments = payment_data['installments'] if 'installments' in payment_data else 1
        body = {
            'customer': {
                'id': payment_data['payer']['decidir_id'],
                'email': payment_data['payer']['email']
            },
            'site_id': self.site_id_cadena if self.marketplace else self.site_id,
            'site_transaction_id': site_transaction_id,
            'token': payment_data['token'],
            'payment_method_id': payment_data['payment_method_id'] if 'payment_method_id' in payment_data else 1,
            'bin': payment_data['bin'],     # primeros 6 digitos de la tarjeta
            'currency': 'ARS',
            'payment_type': 'single',
            'establishment_name': self.merchant_name,
            'sub_payments': [],
            'amount': total_amount, # En cualquier tipo de payment
            'installments': installments,
        }
        if 'application_fee' in payment_data and self.marketplace:
            application_fee = round(payment_data['application_fee'], 2) * 100
            body['payment_type'] = 'distributed'
            body['sub_payments'] = [
                {
                    'site_id': self.site_id_cadena,
                    'installments': installments,
                    'amount': application_fee
                }, {
                    'site_id': self.site_id,
                    'installments': installments,
                    'amount': total_amount - application_fee
                }
            ]
        elif 'application_fee' in payment_data and not self.marketplace:
            # No se puede hacer split si marketplace = False
            return self.error(message='application_fee_not_expected')

        # Si viene informacion adicional
        additional_information = [
            'establishment_name'
        ]
        for item in additional_information:
            if item in payment_data.keys():
                body[item] = payment_data[item]

        # Se genera la request
        r = requests.post(self.url + '/payments', headers=self.headers, json=body)
        response = r.json()
        if 'status' in response and 'site_transaction_id' in response:
            if 'card' in payment_data:
                response['card'] = payment_data['card']
            return self.ok(response=_format(response))
        return self.error(response=response, status=r.status_code)

    def get(self, payment_id: str):
        filters = {
            'siteOperationId': payment_id,
            'expand': 'card_data'
        }
        r = requests.get(self.url + '/payments', params=filters, headers=self.headers)
        list_results = r.json()['results']
        if len(list_results) == 1:
            return self.ok(response=_format(list_results[0]), status=r.status_code)
        else:
            return self.error(message='error_pago_no_encontrado', status=r.status_code)

    def search(self, filters: dict) -> dict:
        # El filter puede ser:
        # offset
        # pageSize
        # siteOperationId
        # merchantId
        # dateFrom
        # dateTo
        # site
        r = requests.get(self.url + '/payments', params=filters, headers=self.headers)
        response = r.json()
        response['results'] = [_format(r) for r in response['results']]
        return self.ok(response=response, status=r.status_code)

    def get_payment_methods(self):
        r = requests.get(self.url + '/payment-methods/1', headers=self.headers)
        return self.ok(response=r.json(), status=r.status_code)



from .Customer import Customer
from .Card import Card
from .CardToken import CardToken
from .Payment import Payment
from .Refund import Refund


class DecidirSDK:
    def __init__(self, merchant: dict, merchant_cadena: dict, marketplace: bool = False):
        if 'credentials' not in merchant or 'decidir' not in merchant['credentials'] or not merchant['credentials']['decidir']['active']:
            self.private_key = None
            return
        credentials_cadena = merchant_cadena['credentials']['decidir']
        credentials_local = merchant['credentials']['decidir']
        self.url = 'https://developers.decidir.com/api/v2'
        self.private_key = credentials_cadena['access_token']
        self.public_key = credentials_cadena['public_key']
        self.merchant_name = merchant['name']
        self.site_id = credentials_local['site_id']
        self.site_id_cadena = credentials_cadena['site_id']
        self.processor = 'decidir'
        self.marketplace = marketplace

    def customer(self):
        return Customer(self.processor, self.url, self.private_key, self.public_key)
    
    def card(self):
        return Card(self.processor, self.url, self.private_key, self.public_key)

    def card_token(self):
        return CardToken(self.processor, self.url, self.private_key, self.public_key)

    def payment(self):
        return Payment(self.processor, self.url, self.private_key, self.public_key, self.merchant_name, self.site_id,
                       self.site_id_cadena, self.marketplace)

    def refund(self):
        return Refund(self.processor, self.url, self.private_key, self.public_key, self.merchant_name, self.site_id,
                       self.site_id_cadena, self.marketplace)

    def ok(self):
        return self.private_key is not None



import requests
import json

from ..MuviBase import MuviBase


class CardToken(MuviBase):
    def __init__(self, processor: str, url: str, private_key: str, public_key: str):
        super().__init__(processor)
        self.url = url
        self.private_key = private_key
        self.public_key = public_key
        self.headers = {
            'apikey': self.private_key,
            'Content-Type': 'application/json',
            'Cache-Control': 'no-cache'
        }

    def create(self, card: dict):
        if 'decidir_id' in card:
            card['id'] = card['decidir_id']

        body = {
            'token': card['id'],
            'security_code': '000' if 'security_code' not in card else card['security_code'],
        }
        self.headers['apikey'] = self.public_key
        r = requests.post(self.url + '/tokens', headers=self.headers, data=json.dumps(body))
        self.headers['apikey'] = self.private_key
        response = r.json()
        if r.status_code > 299:
            return self.error(response=response, message=response['error_type'], status=r.status_code)
        return self.ok(response=response, status=r.status_code)



import mercadopago
from .Customer import Customer
from .Card import Card
from .CardToken import CardToken
from .Payment import Payment
from .Refund import Refund


class MercadoPagoSDK:
    def __init__(self, merchant: dict, marketplace: bool = False):
        active = True
        if 'credentials' in merchant and 'mercadopago' in merchant['credentials'] and not marketplace:
            active = merchant['credentials']['mercadopago']['active']
            access_token = merchant['credentials']['mercadopago']['access_token'] if active else None
        elif 'credentials' in merchant and 'mp_marketplace' in merchant['credentials'] and marketplace:
            active = 'mp_marketplace' in merchant['credentials'] and \
                     ('active' not in merchant['credentials']['mp_marketplace'] or
                      merchant['credentials']['mp_marketplace']['active'])
            access_token = merchant['credentials']['mp_marketplace']['access_token'] if active else None
        else:
            if marketplace:
                access_token = merchant['keys_marketplace']['access_token'] if 'keys_marketplace' in merchant else None
            else:
                access_token = merchant['keys']['access_token'] if 'keys' in merchant else None
            active = access_token is not None

        self.sdk = mercadopago.SDK(access_token) if active else None
        self.merchant_id = merchant['_id']
        self.merchant_name = merchant['name']
        self.processor = 'mercadopago'

    def customer(self):
        return Customer(self.processor, self.sdk)
    
    def card(self):
        return Card(self.processor, self.sdk)

    def card_token(self):
        return CardToken(self.processor, self.sdk)
    
    def payment(self):
        return Payment(self.processor, self.sdk, self.merchant_id)

    def refund(self):
        return Refund(self.processor, self.sdk)

    def ok(self):
        return self.sdk is not None



import mercadopago

from ..MuviBase import MuviBase


class CardToken(MuviBase):
    def __init__(self, processor: str, sdk: mercadopago.SDK):
        super().__init__(processor)
        self.sdk = sdk

    def create(self, card: dict):
        if 'mercadopago_id' in card:
            card['id'] = card['mercadopago_id']

        data = {
            'card_id': card['id']
        }
        try:
            card_token_response = self.sdk.card_token().create(data)
        except:
            return self.error(
                response='error_cant_create_token',
                status=400
            )
        if card_token_response['status'] > 299:
            return self.error(response=card_token_response['response'], status=card_token_response['status'])

        # Para estandarizar la response con decidir
        if 'first_six_digits' in card:
            card_token_response['response']['bin'] = card['first_six_digits']
        elif 'bin' in card:
            card_token_response['response']['bin'] = card['bin']

        for f in ['last_four_digits', 'expiration_month', 'expiration_year', 'cardholder']:
            card_token_response['response'][f] = card[f] if f in card else None

        return self.ok(response=card_token_response['response'], status=card_token_response['status'])



import requests
import json

from ..MuviBase import MuviBase
from .Payment import Payment


class Refund(MuviBase):
    def __init__(self,
                 processor: str,
                 url: str,
                 private_key: str,
                 public_key: str,
                 merchant_name: str,
                 site_id: str,
                 site_id_cadena: str,
                 marketplace: bool):
        super().__init__(processor)
        self.url = url
        self.private_key = private_key
        self.public_key = public_key
        self.merchant_name = merchant_name
        self.site_id = site_id
        self.site_id_cadena = site_id_cadena
        self.marketplace = marketplace
        self.headers = {
            'apikey': self.private_key,
            'Content-Type': 'application/json',
            'Cache-Control': 'no-cache'
        }

    def create(self, payment_id: str, amount: float = None) -> dict:
        # Busco el pago
        payment = Payment(self.processor, self.url, self.private_key, self.public_key, self.merchant_name, self.site_id, self.site_id_cadena, self.marketplace)

        response_get = payment.get(payment_id)
        if response_get['status'] < 400:
            payment_id_decidir = response_get['response']['payment_id']
        else:
            return {'status':404,'response':'payment_not_found'}

        data = {
            'amount': round(amount, 2) * 100 if amount else amount
        }
        r = requests.post(self.url + f'/payments/{payment_id_decidir}/refunds', headers=self.headers, data=json.dumps(data))
        if r.status_code < 400:
            response = r.json()
            response['amount'] = response['amount'] / 100
            return self.ok(response=response, status=r.status_code)
        return self.error(response=r.json(), status=r.status_code)
